package pwc.taxtech.atms.vat.service.impl.report.functions;

import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.vat.entity.*;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 *  根据报表项目与科目映射取出科目代码，再通过入参进行TB表数据计算
 */
public class TBM extends FunctionBase implements FreeRefFunction {
    public TBM(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 4) {
            return null;
        }

        int evenType = getIntParam(args[0], ec);//事件类型，1：利润映射，2：资产映射
        String mappingName = getStringParam(args[1], ec);//映射名称
        int calculateType = getIntParam(args[2], ec);//计算类型，当取利润时候，1：贷方-借方，2：借方-贷方。当取资产时，1：正数，2：负数
        int sourceDataType = getIntParam(args[3], ec);//取值数据源
        String formulaExpression = "TBM(" + evenType + ",\""+mappingName+ "\","+calculateType+","+sourceDataType+ ")";

        List<ReportCellDataSourceDto> dataSource = Lists.newArrayList();

        List<String> segment3List = getSegment3List(evenType,mappingName);
        if(CollectionUtils.isEmpty(segment3List)){
            return NumberEval.ZERO;
        }
        double result = 0.00;
        if (sourceDataType == 1) {
            result = countForTrialBalance(evenType,calculateType,segment3List,dataSource,formulaContext.getPeriod(),formulaContext.getYear(),formulaContext.getOrganizationId());
        } else if (sourceDataType == 2) {
            result = countForAdjBalance(evenType,calculateType,segment3List,dataSource,formulaContext.getPeriod(),formulaContext.getYear(),formulaContext.getOrganizationId());
        } else if (sourceDataType == 3) {
            result = countForTrialFinalBalance(evenType,calculateType,segment3List,dataSource,formulaContext.getPeriod(),formulaContext.getYear(),formulaContext.getOrganizationId());
        } else {
            return NumberEval.ZERO;
        }
        Long dataSoureId = saveDataSource(ec, Collections.singletonList(dataSource),
                FormulaDataSourceDetailType.FormulaDataSourceDto,
                new BigDecimal(result), formulaContext.getPeriod(), formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
        saveFormulaBlock(formulaContext.getPeriod(), ec, formulaExpression, new BigDecimal(result), dataSoureId, formulaContext.getProjectId());
        return new NumberEval(result);

    }

    public List<String> getSegment3List(int eventType,String mappingName){
        TrialBalanceMappingExample example = new TrialBalanceMappingExample();
        if(eventType == 1){
            example.createCriteria().andPlTypeEqualTo(mappingName);
        }else if(eventType == 2){
            example.createCriteria().andBsTypeEqualTo(mappingName);
        }
        List<TrialBalanceMapping> dataList = SpringContextUtil.trialBalanceMappingMapper.selectByExample(example);
        return dataList.stream()
                .map(o -> o.getSegment3()).collect(Collectors.toList());
    }

    private int pardePeriod(int period, int year) {
        return Integer.parseInt("" + year + (period > 9 ? period : ("0" + period)));
    }
    private double countForTrialBalance(int eventType,int calculateType,List<String> segment3List, List<ReportCellDataSourceDto> contain, int period, int year, String orgId) {
        Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(orgId);
        TrialBalanceExample glBalanceExample = new TrialBalanceExample();
        TrialBalanceExample.Criteria c1 = glBalanceExample.createCriteria().andSegment3In(segment3List)
                .andPeriodEqualTo(pardePeriod(period, year));
        if (organization != null) {
            c1.andOrganizationIdEqualTo(organization.getId());
        }
        List<TrialBalance> list = SpringContextUtil.trialBalanceMapper.selectByExample(glBalanceExample);
        if (CollectionUtils.isEmpty(list)) {
            return 0.0;
        }
        BigDecimal amount = new BigDecimal(0);
        for (TrialBalance balance : list) {
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            if(eventType == 1 ){
                if(calculateType == 1){
                    dto.setAmount(balance.getPeriodCrBeq().subtract(balance.getPeriodDrBeq()));
                }else if(calculateType == 2){
                    dto.setAmount(balance.getPeriodDrBeq().subtract(balance.getPeriodCrBeq()));
                }
            }else if(eventType == 2){
                if(calculateType == 1){
                    dto.setAmount(balance.getEndBalBeq());
                }else if(calculateType == 2){
                    dto.setAmount(balance.getEndBalBeq().multiply(new BigDecimal(-1)));
                }
            }
            amount = amount.add(dto.getAmount()==null?(new BigDecimal(0)):dto.getAmount());
            dto.setPeriod(period);
            dto.setIsOnlyManualInput(Boolean.FALSE);
            dto.setName(Constant.DataSourceName.ReportDataSource);
            dto.setType(FormulaDataSourceType.TrialBalanceSource.getCode());
            contain.add(dto);
        }
        return amount.doubleValue();
    }

    private double countForAdjBalance(int eventType,int calculateType,List<String> segment3List, List<ReportCellDataSourceDto> contain, int period, int year, String orgId) {
        Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(orgId);
        AdjustmentTableExample glBalanceExample = new AdjustmentTableExample();
        AdjustmentTableExample.Criteria c1 = glBalanceExample.createCriteria().andSegment3In(segment3List)
                .andPeriodEqualTo(pardePeriod(period, year));
        if (organization != null) {
            c1.andOrganizationIdEqualTo(organization.getId());
        }
        List<AdjustmentTable> list = SpringContextUtil.adjustmentTableMapper.selectByExample(glBalanceExample);
        if (CollectionUtils.isEmpty(list)) {
            return 0.0;
        }
        BigDecimal amount = new BigDecimal(0);
        for (AdjustmentTable balance : list) {
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            if(eventType == 1 ){
                if(calculateType == 1){
                    dto.setAmount(balance.getPeriodCrBeq().subtract(balance.getPeriodDrBeq()));
                }else if(calculateType == 2){
                    dto.setAmount(balance.getPeriodDrBeq().subtract(balance.getPeriodCrBeq()));
                }
            }else if(eventType == 2){
                if(calculateType == 1){
                    dto.setAmount(balance.getEndBalBeq());
                }else if(calculateType == 2){
                    dto.setAmount(balance.getEndBalBeq().multiply(new BigDecimal(-1)));
                }
            }
            amount = amount.add(dto.getAmount()==null?(new BigDecimal(0)):dto.getAmount());
            dto.setPeriod(period);
            dto.setIsOnlyManualInput(Boolean.FALSE);
            dto.setName(Constant.DataSourceName.ReportDataSource);
            dto.setType(FormulaDataSourceType.TrialBalanceSource.getCode());
            contain.add(dto);
        }
        return amount.doubleValue();
    }

    private double countForTrialFinalBalance(int eventType,int calculateType,List<String> segment3List, List<ReportCellDataSourceDto> contain, int period, int year, String orgId) {
        Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(orgId);
        TrialBalanceFinalExample glBalanceExample = new TrialBalanceFinalExample();
        TrialBalanceFinalExample.Criteria c1 = glBalanceExample.createCriteria().andSegment3In(segment3List)
                .andPeriodEqualTo(pardePeriod(period, year));
        if (organization != null) {
            c1.andOrganizationIdEqualTo(organization.getId());
        }
        List<TrialBalanceFinal> list = SpringContextUtil.trialBalanceFinalMapper.selectByExample(glBalanceExample);
        if (CollectionUtils.isEmpty(list)) {
            return 0.0;
        }
        BigDecimal amount = new BigDecimal(0);
        for (TrialBalanceFinal balance : list) {
            ReportCellDataSourceDto dto = new ReportCellDataSourceDto();
            if(eventType == 1 ){
                if(calculateType == 1){
                    dto.setAmount(balance.getPeriodCrBeq().subtract(balance.getPeriodDrBeq()));
                }else if(calculateType == 2){
                    dto.setAmount(balance.getPeriodDrBeq().subtract(balance.getPeriodCrBeq()));
                }
            }else if(eventType == 2){
                if(calculateType == 1){
                    dto.setAmount(balance.getEndBalBeq());
                }else if(calculateType == 2){
                    dto.setAmount(balance.getEndBalBeq().multiply(new BigDecimal(-1)));
                }
            }
            amount = amount.add(dto.getAmount()==null?(new BigDecimal(0)):dto.getAmount());
            dto.setPeriod(period);
            dto.setIsOnlyManualInput(Boolean.FALSE);
            dto.setName(Constant.DataSourceName.ReportDataSource);
            dto.setType(FormulaDataSourceType.TrialBalanceSource.getCode());
            contain.add(dto);
        }
        return amount.doubleValue();
    }
}
